/***************************************************************************
    ep.cxx

    Copyright (c) 2002-2003  Vladimir Toncar

    The contents of this file are subject to the Mozilla Public License
    Version 1.1 (the "License"); you may not use this file except in
    compliance with the License. You may obtain a copy of the License at
    http://www.mozilla.org/MPL/

    Software distributed under the License is distributed on an "AS IS"
    basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
    See the License for the specific language governing rights and
    limitations under the License.

    The Original Code is oh323tut.

    The Initial Developer of the Original Code is Vladimir Toncar.
    All Rights Reserved.

    Contributor(s): ______________________________________.

 ***************************************************************************/

#include <ptlib.h>
#include "ep.h"
#include "nullchan.h"
#include "wavchan.h"


// ***********************************************************************

MyEndPoint::MyEndPoint(const ProgConf & conf) : progConf(conf)  
{}


// ***********************************************************************

MyEndPoint::~MyEndPoint() 
{}


// ***********************************************************************

void MyEndPoint::OnConnectionEstablished(H323Connection & connection, 
						const PString & token)
{
	PTRACE(1, "Connection established, token is " << token);
} 


// ***********************************************************************

void MyEndPoint::OnConnectionCleared(H323Connection &connection, 
						const PString &token)
{
	PTRACE(1, "Connection cleared, token is " << token);
} 


// ***********************************************************************

H323Connection::AnswerCallResponse MyEndPoint::OnAnswerCall( 
						H323Connection & connection,
						const PString & caller, 
						const H323SignalPDU &, 
						H323SignalPDU &)
{
	PTRACE(1, "Accepting connection from " << caller);
	return H323Connection::AnswerCallNow;
} 


// ***********************************************************************

bool MyEndPoint::Init() 
{
	// Set user name
	if (!progConf.user.IsEmpty()) 
		SetLocalUserName(progConf.user); 
	
	//DisableFastStart(true);	
	//DisableH245Tunneling(true);
	//DisableH245inSetup(true);	
		
	// Codecs
	SetCapability(0, 0, new H323_GSM0610Capability); 
	SetCapability(0, 0, new H323_G711Capability(H323_G711Capability::muLaw) );
	SetCapability(0, 0, new H323_G711Capability(H323_G711Capability::ALaw) );
	SetCapability(0, 0, new H323_LPC10Capability(*this) ); 
	
	AddAllUserInputCapabilities(0, 1); 
					
	PTRACE(1, "Capabilities:\n" << setprecision(2) << capabilities); 

	// Start H323 Listener
	PIPSocket::Address addr = INADDR_ANY; 
	H323ListenerTCP *listener = new H323ListenerTCP(*this, addr,
							progConf.port);
	if (listener == NULL || !StartListener(listener))
	{	PError << "Unable to start H323 Listener at port "
				<< progConf.port << endl;
		if (listener != NULL)
			delete listener;
		return false;
	} 

	// Gatekeeper registration
	bool gkResult = false; 
	switch (progConf.gkMode)
	{
	case ProgConf::NoGatekeeper: 
		gkResult = true;
		break;
	case ProgConf::RegisterWithGatekeeper:
		gkResult = UseGatekeeper(progConf.gkAddr, progConf.gkId);
		break;
	default:
		break;
	}

	if (!gkResult)
	{
		PError << "Failed to register with gatekeeper" << endl; 
		return false;
	} 
		
	return true;
} 


// ***********************************************************************

BOOL MyEndPoint::OpenAudioChannel(H323Connection &connection, 
						BOOL isEncoding,
						unsigned bufferSize, 
						H323AudioCodec &codec)
{
	//codec.SetSilenceDetectionMode(H323AudioCodec::NoSilenceDetection); 
	if (isEncoding) 
	{	// send audio direction
		WavChannel *ch = new WavChannel(progConf.fileName, connection);
		return codec.AttachChannel(ch, true); 
	}
	else
	{	// receive audio direction
		NullChannel *ch = new NullChannel();
		return codec.AttachChannel(ch, true);
	}
	
	return false;
} 


// ***********************************************************************

BOOL MyEndPoint::OnStartLogicalChannel(H323Connection & connection, 
						H323Channel & channel)
{
	PString dir;
	switch (channel.GetDirection())
	{
	case H323Channel::IsTransmitter :
		dir = "sending";
		break;
	case H323Channel::IsReceiver :
		dir = "receiving";
		break;
	default :
		break;
	}

	PTRACE(1, "Started logical channel " << dir << " " 
		<< channel.GetCapability() );
	return true;
} 


// ***********************************************************************

